/*
An Arduino Morse Beacon
Adaptation of a work  by Mark VandeWettering 
Prepared for www.electroschematics.com
Authored & Tested by T.K.Hareendran
Modified: Fernando Fernandez de Villegas (EB3EMD)
*/

/////////////// Simple Arduino CW Beacon /////////////////
//
//        Written by Mark VandeWettering K6HX
//

//////// CW section ////////

struct t_mtab { char c, pat; } ;
struct t_mtab morsetab[ ] = {
 	{'.', 106},
	{',', 115},
	{'?', 76},
	{'/', 41},
	{'A', 6},
	{'B', 17},
	{'C', 21},
	{'D', 9},
	{'E', 2},
	{'F', 20},
	{'G', 11},
	{'H', 16},
	{'I', 4},
	{'J', 30},
	{'K', 13},
	{'L', 18},
	{'M', 7},
	{'N', 5},
	{'O', 15},
	{'P', 22},
	{'Q', 27},
	{'R', 10},
	{'S', 8},
	{'T', 3},
	{'U', 12},
	{'V', 24},
	{'W', 14},
	{'X', 25},
	{'Y', 29},
	{'Z', 19},
	{'1', 62},
	{'2', 60},
	{'3', 56},
	{'4', 48},
	{'5', 32},
	{'6', 33},
	{'7', 35},
	{'8', 39},
	{'9', 47},
	{'0', 63}
} ;
#define N_MORSE  (sizeof(morsetab)/sizeof(morsetab[0]))
#define SPEED (7)
#define DOTLEN (1200/SPEED)
#define DASHLEN (3*(1200/SPEED))

////////////// ARDUINO PINS CONFIGURATION  ////////////////

int SIGkey = 13 ;  // Keyer
int SIGptt = 12 ;  // PTT control
int SIGtone = 11 ; // CW Tone
int freq = 1200 ;  // Tone Frequency

//////////////// CW GENERATION ROUTINES  //////////////////

void dash() {
  digitalWrite(SIGkey, HIGH) ;
  tone(SIGtone, freq);
  delay(DASHLEN);
  digitalWrite(SIGkey, LOW) ;
  noTone (SIGtone);
  delay(DOTLEN) ;
}
////////////////////////////
void dit() {
  digitalWrite(SIGkey, HIGH) ;
  tone(SIGtone, freq);
  delay(DOTLEN);
  digitalWrite(SIGkey, LOW) ;
  noTone (SIGtone);
  delay(DOTLEN);
}
////////////////////////////
void send(char c) {
  int i ;
  if (c == ' ') {
    Serial.print(c) ;
    delay(7*DOTLEN) ;
    return ;
  }
  for (i=0; i<N_MORSE; i++) {
    if (morsetab[i].c == c) {
      unsigned char p = morsetab[i].pat ;
      Serial.print(morsetab[i].c) ;
      while (p != 1) {
          if (p & 1)
            dash() ;
          else
            dit() ;
          p = p / 2 ;
      }
      delay(2*DOTLEN) ;
      return ;
    }
  }
  /* if we drop off the end, then we send a space */
  Serial.print("?") ;
}
////////////////////////////
void sendmsg(char *str) {
  while (*str)
    send(*str++) ;
  Serial.println("");
}

////////////////////// S E T U P //////////////////////////

void setup() {
  pinMode(SIGkey, OUTPUT) ; // Signal Output CW Pin
  pinMode(SIGptt, OUTPUT) ; // Signal control PTT Pin 
  Serial.begin(9600) ;
  Serial.println("An Arduino Morse Beacon") ;
  Serial.println("") ;
}

////////////////////// L O O P ////////////////////////////

void loop() {
  delay (1500); // Not inmediate transmission after a power-on or reset
  digitalWrite(SIGptt, HIGH); // PTT ON
  delay (150);
  for (int j = 0; j < 4; j++){ // Send advertiment for CW Tone transmision
  tone(SIGtone, 1200); 
  delay (150);
  tone(SIGtone, 1000);
  delay (150);   
  }
  noTone (SIGtone);
  delay (700);
  sendmsg("ESCOM BEACON") ; // Custom Message, in capital letters (Send Message)
  digitalWrite(SIGptt, LOW); // PTT OFF
  digitalWrite(SIGkey, LOW); // Ensure Keyer OFF
  delay(212000) ; // Delay in miliseconds
}
