/*
An Arduino Morse Beacon
Adaptation of a work by Mark VandeWettering K6HX
Adapted for use in chip ATTiny85: Fernando Fernandez de Villegas (EB3EMD)
*/

/////////// CW SECTION (by Mark VandeWettering) ///////////

struct t_mtab { char c, pat; } ;
struct t_mtab morsetab[ ] = {
 	{'.', 106},
	{',', 115},
	{'?', 76},
	{'/', 41},
	{'A', 6},
	{'B', 17},
	{'C', 21},
	{'D', 9},
	{'E', 2},
	{'F', 20},
	{'G', 11},
	{'H', 16},
	{'I', 4},
	{'J', 30},
	{'K', 13},
	{'L', 18},
	{'M', 7},
	{'N', 5},
	{'O', 15},
	{'P', 22},
	{'Q', 27},
	{'R', 10},
	{'S', 8},
	{'T', 3},
	{'U', 12},
	{'V', 24},
	{'W', 14},
	{'X', 25},
	{'Y', 29},
	{'Z', 19},
	{'1', 62},
	{'2', 60},
	{'3', 56},
	{'4', 48},
	{'5', 32},
	{'6', 33},
	{'7', 35},
	{'8', 39},
	{'9', 47},
	{'0', 63}
} ;

#define N_MORSE (sizeof(morsetab)/sizeof(morsetab[0]))
#define SPEED (8)
#define DOTLEN (1200/SPEED)
#define DASHLEN (3.5*(1200/SPEED))

//////////////// ATTINY CONFIGURATION  ////////////////////

int SIGkey = 0 ;  // terminal 5 ATTiny85 , Keyer
int SIGptt = 4 ;  // terminal 3 ATTiny85 , PTT control
int SIGtone = 2 ; // terminal 7 ATTiny85 , CW Tone
int freq = 1200 ; // CW Tone Frequency

//////////////// CW GENERATION ROUTINES  //////////////////

void dash() {
  digitalWrite(SIGkey, HIGH) ;
  tone(SIGtone, freq);
  delay(DASHLEN);
  digitalWrite(SIGkey, LOW) ;
  noTone (SIGtone);
  delay(DOTLEN) ;
}
////////////////////////////
void dit() {
  digitalWrite(SIGkey, HIGH) ;
  tone(SIGtone, freq);
  delay(DOTLEN);
  digitalWrite(SIGkey, LOW) ;
  noTone (SIGtone);
  delay(DOTLEN);
}
////////////////////////////
void send(char c) {
  int i ;
  if (c == ' ') {
    delay(7*DOTLEN) ;
    return ;
  }
  for (i=0; i<N_MORSE; i++) {
    if (morsetab[i].c == c) {
      unsigned char p = morsetab[i].pat ;
      while (p != 1) {
          if (p & 1)
            dash() ;
          else
            dit() ;
          p = p / 2 ;
      }
      delay(2*DOTLEN) ;
      return ;
    }
  }
  /* if we drop off the end, then we send a space */
}
////////////////////////////
void sendmsg(char *str) {
  while (*str)
    send(*str++) ;
}

////////////////////// S E T U P //////////////////////////

void setup() {
  pinMode(SIGkey, OUTPUT) ; // Signal Output CW Pin
  pinMode(SIGptt, OUTPUT) ; // Signal control PTT Pin
  digitalWrite(SIGptt, LOW);
  digitalWrite(SIGkey, LOW);
  // pins not used is recommended to ensure that these pins have a defined level
  pinMode(1, INPUT) ;      //  terminal 6 ATTiny85 (not used)
  digitalWrite(1, HIGH);
  pinMode(3, INPUT) ;      //  terminal 2 ATTiny85 (not used)
  digitalWrite(3, HIGH);
}

////////////////////// L O O P ////////////////////////////

void loop() {
  delay (1500);  // Not inmediate transmission after a power-on or reset
  digitalWrite(SIGkey, LOW); // Ensure initial Keyer OFF
  digitalWrite(SIGptt, HIGH); // PTT ON
  delay (150);
  for (int j = 0; j < 4; j++){ // Send advertiment for CW Tone transmision
  tone(SIGtone, 1200); 
  delay (125);
  tone(SIGtone, 1000);
  delay (125);   
  }
  noTone (SIGtone);
  delay (1000);
  sendmsg("CQ CQ CQ") ; // Custom Message (Send Message)
  digitalWrite(SIGptt, LOW); // PTT OFF
  digitalWrite(SIGkey, LOW); // Ensure Keyer OFF
  delay(212000) ;
}
